/*
	-> Este é um arquivo base para desenvolvimento na camada de Infraestrutura;
	-> Codifique em inglês utilizando os termos definidos no glossário;
	-> Qualquer dúvida contate um AGES 3.
*/
package infrastructure

import (
	"context"
	"errors"
	"fmt"
	"institutionMS/layers/entity"
	"log"
	"time"

	"go.mongodb.org/mongo-driver/bson"
	"go.mongodb.org/mongo-driver/bson/primitive"
	"go.mongodb.org/mongo-driver/mongo"
	"go.mongodb.org/mongo-driver/mongo/options"
)

// MongoDB is a struct that defines a Mongo DB connection at infrastructure layer
type MongoDB struct {
	Username   string
	Password   string
	Host       string
	Port       string
	client     *mongo.Client
	ctx        context.Context
	collection *mongo.Collection
}

// setUri sets the connection string for MongoDB connection
func (mongodb *MongoDB) setUri() (uri string) {
	uri = "mongodb://" + mongodb.Host + ":" + mongodb.Port
	return
}

// setCredentials sets credentials for MongoDB connection
func (mongodb *MongoDB) setCredentials() (credentials options.Credential) {
	credentials = options.Credential{
		Username: mongodb.Username,
		Password: mongodb.Password,
	}
	return
}

// Connect creates a connection to MongoDB
func (mongodb *MongoDB) Connect() {
	var err error

	mongodb.ctx, _ = context.WithTimeout(context.Background(), 60*time.Second)
	opts := options.Client().ApplyURI(mongodb.setUri()).SetAuth(mongodb.setCredentials())
	mongodb.client, err = mongo.Connect(mongodb.ctx, opts)

	if err != nil {
		log.Fatal(error.Error(err))
	}
}

// Collection sets a collection at a database for MongoDB connection
func (mongodb *MongoDB) Collection(database string, collection string) {
	mongodb.collection = mongodb.client.Database(database).Collection(collection)
}

// Load loads a Institution from MongoDB through id
func (mongodb MongoDB) Load(id string) (institution *entity.Institution) {
	opts := options.FindOne().SetSort(bson.D{{"name", 1}})
	objectId, _ := primitive.ObjectIDFromHex(id)

	err := mongodb.collection.FindOne(
		mongodb.ctx,
		bson.D{{"_id", objectId}},
		opts,
	).Decode(&institution)

	if err != nil {
		if err == mongo.ErrNoDocuments {
			return
		}
		log.Fatal(error.Error(err))
	}
	return
}

// Save saves a Institution at MongoDB and returns its given id
func (mongodb MongoDB) Save(institution entity.Institution) (id string) {
	res, err := mongodb.collection.InsertOne(mongodb.ctx, institution)

	if err != nil {
		log.Fatal(error.Error(err))
	}

	if res != nil {
		id = fmt.Sprintf("%v", res.InsertedID)
	}

	objectId, _ := res.InsertedID.(primitive.ObjectID)
	return objectId.Hex()
}

// Disconnect closes MongoDB client using timeout context
func (mongodb MongoDB) Disconnect() {
	err := mongodb.client.Disconnect(mongodb.ctx)

	if err != nil {
		log.Fatal(error.Error(err))
	}
}

// Update a Institution from MongoDB through id
func (mongodb MongoDB) Update(id string, institution *entity.Institution) (updated *entity.Institution, err error) {
	objectId, err := primitive.ObjectIDFromHex(id)
	if err != nil {
		return nil, errors.New("Parâmetro inválido!")
	}

	filter := bson.D{{"_id", objectId}}
	update := bson.D{{"$set",
		bson.D{
			{"id", institution.ID},
			{"isvalid", institution.IsValid},
			{"password", institution.Password},
			{"name", institution.Name},
			{"email", institution.Email},
			{"description", institution.Description},
			{"location", institution.Location},
		},
	}}
	res, err := mongodb.collection.UpdateOne(mongodb.ctx, filter, update)

	if err != nil {
		log.Fatal(error.Error(err))
	}
	if res.MatchedCount == 0 {
		return nil, errors.New("Instituição não localizada!")
	}

	institution.ID = objectId
	return institution, nil
}
